PlaceableEnergyGenerator = {
    prerequisitesPresent = function(specializations)
        return SpecializationUtil.hasSpecialization(PlaceableIncomePerHour, specializations) and 
            SpecializationUtil.hasSpecialization(PlaceableProductionPoint, specializations)
    end,

}
PlaceableEnergyGenerator.MOD_NAME = g_currentModName
PlaceableEnergyGenerator.NAME = ".placeableEnergyGenerator"
PlaceableEnergyGenerator.SPEC_NAME = PlaceableEnergyGenerator.MOD_NAME .. PlaceableEnergyGenerator.NAME
PlaceableEnergyGenerator.KEY = "." .. PlaceableEnergyGenerator.MOD_NAME .. PlaceableEnergyGenerator.NAME

function PlaceableEnergyGenerator.registerFunctions(placeableType)
    -- SpecializationUtil.registerFunction(placeableType, "onSpeedCameraTriggerCallback", PlaceableEnergyGenerator.onSpeedCameraTriggerCallback)
    
end

function PlaceableEnergyGenerator.registerEventListeners(placeableType)
    SpecializationUtil.registerEventListener(placeableType, "onLoad", PlaceableEnergyGenerator)
    SpecializationUtil.registerEventListener(placeableType, "onMinuteChanged", PlaceableEnergyGenerator)
end

function PlaceableEnergyGenerator.registerOverwrittenFunctions(placeableType)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "getNeedMinuteChanged", PlaceableEnergyGenerator.getNeedMinuteChanged)
end

function PlaceableEnergyGenerator.registerXMLPaths(schema, basePath)
    schema:setXMLSpecializationType("PlaceableEnergyGenerator")
    basePath = basePath .. ".placeableEnergyGenerator"
    schema:register(XMLValueType.STRING, basePath .. ".productions(?).production#id")
    schema:register(XMLValueType.STRING, basePath .. ".productions(?).production#fillType")
    schema:setXMLSpecializationType()
end

function PlaceableEnergyGenerator:onLoad(savegame)
    self.spec_placeableEnergyGenerator = self["spec_" .. PlaceableEnergyGenerator.SPEC_NAME]
    local spec = self.spec_placeableEnergyGenerator
    spec.productions = {}
    local key = "placeable.placeableEnergyGenerator"
    self.xmlFile:iterate(key .. ".productions", function (index, productionKey)
        local id = self.xmlFile:getValue(productionKey .. ".production#id")
        local fillTypeStr = self.xmlFile:getValue(productionKey .. ".production#fillType")
        spec.productions[id] = g_fillTypeManager:getFillTypeIndexByName(fillTypeStr)
    end)

    -- spec.trigger = self.xmlFile:getValue(key .. "#triggerNode", nil, self.components, self.i3dMappings)

    -- if spec.trigger == nil then
    --     Logging.xmlError(self.xmlFile, "Missing vehicle triggerNode for speed camera")

    --     return
    -- end

    -- addTrigger(spec.trigger, "onSpeedCameraTriggerCallback", self)

    -- spec.flashLight = self.xmlFile:getValue(key .. "#flashLightNode", nil, self.components, self.i3dMappings)
    -- spec.triggerMarkers = self.xmlFile:getValue(key .. "#triggerMarkersNode", nil, self.components, self.i3dMappings)
    -- spec.speedLimit = self.xmlFile:getInt(key .. ".speedLimit", 50)
    -- -- free limit
    -- spec.speedLimit = spec.speedLimit + math.min(5, spec.speedLimit * 0.1)
    -- spec.costPerKMH = self.xmlFile:getInt(key .. ".costPerKMH", 50)
    -- spec.ownerGetsMoney = self.xmlFile:getBool(key .. ".ownerGetsMoney", false)
    -- spec.showBlinkingWarning = self.xmlFile:getBool(key .. "#showBlinkingWarning", false)
    -- spec.count = 0
    -- spec.timer = 0
    -- spec.flashed = false

    -- if spec.flashLight ~= nil then
    --     setVisibility(spec.flashLight, false)
    -- end

    -- if spec.triggerMarkers ~= nil then
    --     setVisibility(spec.triggerMarkers, true)
    -- end

    -- spec.activatable = PlaceableEnergyGeneratorActivateable.new(self)
end

function PlaceableEnergyGenerator:saveToXMLFile(xmlFile, key, usedModNames)
    local spec = self.spec_placeableEnergyGenerator
    -- local keyParts = string.split(key, ".")
    -- key = keyParts[1] .. "." .. keyParts[2] .. "." .. keyParts[4]

    -- xmlFile:setInt(key .. ".speedLimit", spec.speedLimit)
    -- xmlFile:setInt(key .. ".costPerKMH", spec.costPerKMH)
    -- xmlFile:setBool(key .. ".ownerGetsMoney", spec.ownerGetsMoney)
end

function PlaceableEnergyGenerator:loadFromXMLFile(xmlFile, key)
    local spec = self.spec_placeableEnergyGenerator
    -- local keyParts = string.split(key, ".")
    -- key = keyParts[1] .. "." .. keyParts[2] .. "." .. keyParts[4]

    -- spec.speedLimit = xmlFile:getInt(key .. ".speedLimit") or spec.speedLimit
    -- spec.costPerKMH = xmlFile:getInt(key .. ".costPerKMH") or spec.costPerKMH
    -- spec.ownerGetsMoney = xmlFile:getBool(key .. ".ownerGetsMoney") or spec.ownerGetsMoney
end

function PlaceableEnergyGenerator:onWriteStream(streamId, connection)
    local spec = self.spec_placeableEnergyGenerator

    -- streamWriteInt32(streamId, spec.speedLimit)
    -- streamWriteInt32(streamId, spec.costPerKMH)
    -- streamWriteBool(streamId, spec.ownerGetsMoney)
end

function PlaceableEnergyGenerator:onReadStream(streamId, connection)
    local spec = self.spec_placeableEnergyGenerator

    -- spec.speedLimit = streamReadInt32(streamId)
    -- spec.costPerKMH = streamReadInt32(streamId)
    -- spec.ownerGetsMoney = streamReadBool(streamId)
end

function PlaceableEnergyGenerator:getNeedMinuteChanged()
    return true
end

function PlaceableEnergyGenerator:onMinuteChanged()
    local spec = self.spec_placeableEnergyGenerator
    local productionPoint = self.spec_productionPoint.productionPoint

    for index, filltype in pairs(spec.productions) do 
        local prod = productionPoint.productionsIdToObj[index]
        for _, input in ipairs(prod.inputs) do
            if input.type == filltype then
                productionPoint.storage:setFillLevel(self:getIncomePerHourFactor() * 100, filltype)
            end
        end
    end

    if g_currentMission ~= nil and g_currentMission.environment ~= nil then
        local minute = g_currentMission.environment.currentMinute
        if minute % 15 == 0 and minute > 0 then 
            self:updateHeadRotation()
        end
    end

    -- if spec.flashed then
    --     spec.timer = spec.timer + dt
    --     self:raiseActive()
    -- end

    -- if spec.timer >= 125 then
    --     spec.flashed = false
    --     spec.timer = 0

    --     if spec.flashLight ~= nil then
    --         setVisibility(spec.flashLight, false)
    --     end
    -- end
end

function PlaceableEnergyGenerator:updateHeadRotation()
    local spec = self.spec_solarPanels

	if spec.headNode ~= nil and g_currentMission ~= nil and g_currentMission.environment ~= nil then
		local sunLight = g_currentMission.environment.lighting.sunLightId
        local dayligdayStart, dayEnd, nightEnd, nightStart = g_currentMission.environment.daylight:getDaylightTimes()
        local time = g_currentMission.environment.currentHour
        if time >= dayligdayStart and time <= dayEnd then
            if sunLight ~= nil then
                local dx, _, dz = localDirectionToWorld(sunLight, 0, 0, 1)
                local headRotation = math.atan2(dx, dz)

                if math.abs(dx) > 0.1 then
                    headRotation = headRotation + spec.headRotationRandom
                    spec.targetRotation = headRotation

                    self:raiseActive()
                end
            end
        end
	end
end
PlaceableSolarPanels.updateHeadRotation = Utils.overwrittenFunction(PlaceableSolarPanels.updateHeadRotation, PlaceableEnergyGenerator.updateHeadRotation)